@@ -5,6 +5,7 @@ use crate::{
     utils::{get_iter_capacity, NoNull},
 };
 pub use arrow::alloc;
+use arrow::array::{ArrayData, PrimitiveArray, PrimitiveBuilder};
 use arrow::array::{ArrayRef, LargeListBuilder};
 use arrow::{array::Array, buffer::Buffer};
 use num::Num;
@@ -52,6 +53,7 @@ impl ChunkedBuilder<bool, BooleanType> for BooleanChunkedBuilder {
             chunks: vec![arr],
             phantom: PhantomData,
             categorical_map: None,
+            ..Default::default()
         }
     }
 }
@@ -70,7 +72,7 @@ where
     T: PolarsPrimitiveType,
     T::Native: Default,
 {
-    array_builder: PrimitiveArrayBuilder<T>,
+    array_builder: PrimitiveBuilder<T>,
     field: Field,
 }
 
@@ -82,13 +84,13 @@ where
     /// Appends a value of type `T` into the builder
     #[inline]
     fn append_value(&mut self, v: T::Native) {
-        self.array_builder.append_value(v)
+        self.array_builder.append_value(v).unwrap()
     }
 
     /// Appends a null slot into the builder
     #[inline]
     fn append_null(&mut self) {
-        self.array_builder.append_null()
+        self.array_builder.append_null().unwrap()
     }
 
     fn finish(mut self) -> ChunkedArray<T> {
@@ -99,6 +101,7 @@ where
             chunks: vec![arr],
             phantom: PhantomData,
             categorical_map: None,
+            ..Default::default()
         }
     }
 }
@@ -109,7 +112,7 @@ where
 {
     pub fn new(name: &str, capacity: usize) -> Self {
         PrimitiveChunkedBuilder {
-            array_builder: PrimitiveArrayBuilder::<T>::new(capacity),
+            array_builder: PrimitiveBuilder::<T>::new(capacity),
             field: Field::new(name, T::get_dtype()),
         }
     }
@@ -163,6 +166,7 @@ impl Utf8ChunkedBuilder {
             chunks: vec![arr],
             phantom: PhantomData,
             categorical_map: None,
+            ..Default::default()
         }
     }
 }
@@ -233,7 +237,11 @@ where
     T: PolarsPrimitiveType,
 {
     fn new_from_slice(name: &str, v: &[T::Native]) -> Self {
-        Self::new_from_iter(name, v.iter().copied())
+        let array_data = ArrayData::builder(T::DATA_TYPE)
+            .len(v.len())
+            .add_buffer(Buffer::from_slice_ref(&v))
+            .build();
+        ChunkedArray::new_from_chunks(name, vec![Arc::new(PrimitiveArray::<T>::from(array_data))])
     }
 
     fn new_from_opt_slice(name: &str, opt_v: &[Option<T::Native>]) -> Self {
@@ -303,6 +311,7 @@ where
             chunks: vec![Arc::new(builder.finish())],
             phantom: PhantomData,
             categorical_map: None,
+            ..Default::default()
         }
     }
 
@@ -347,7 +356,7 @@ pub struct ListPrimitiveChunkedBuilder<T>
 where
     T: PolarsPrimitiveType,
 {
-    pub builder: LargeListBuilder<PrimitiveArrayBuilder<T>>,
+    pub builder: LargeListBuilder<PrimitiveBuilder<T>>,
     field: Field,
 }
 
@@ -359,6 +368,7 @@ macro_rules! finish_list_builder {
             chunks: vec![arr],
             phantom: PhantomData,
             categorical_map: None,
+            ..Default::default()
         }
     }};
 }
@@ -367,7 +377,7 @@ impl<T> ListPrimitiveChunkedBuilder<T>
 where
     T: PolarsPrimitiveType,
 {
-    pub fn new(name: &str, values_builder: PrimitiveArrayBuilder<T>, capacity: usize) -> Self {
+    pub fn new(name: &str, values_builder: PrimitiveBuilder<T>, capacity: usize) -> Self {
         let builder = LargeListBuilder::with_capacity(values_builder, capacity);
         let field = Field::new(name, DataType::List(T::get_dtype().to_arrow()));
 
@@ -377,7 +387,7 @@ where
     pub fn append_slice(&mut self, opt_v: Option<&[T::Native]>) {
         match opt_v {
             Some(v) => {
-                self.builder.values().append_slice(v);
+                self.builder.values().append_slice(v).unwrap();
                 self.builder.append(true).expect("should not fail");
             }
             None => {
@@ -409,7 +419,7 @@ where
     #[inline]
     fn append_null(&mut self) {
         let builder = self.builder.values();
-        builder.append_null();
+        builder.append_null().unwrap();
         self.builder.append(true).unwrap();
     }
 
@@ -417,21 +427,29 @@ where
     fn append_series(&mut self, s: &Series) {
         let builder = self.builder.values();
         let arrays = s.chunks();
-        for a in arrays {
-            let values = a.get_values::<T>();
-            // we would like to check if array has no null values.
-            // however at the time of writing there is a bug in append_slice, because it does not update
-            // the null bitmap
-            if s.null_count() == 0 {
-                builder.append_slice(values);
-            } else {
-                values.iter().enumerate().for_each(|(idx, v)| {
-                    if a.is_valid(idx) {
-                        builder.append_value(*v);
-                    } else {
-                        builder.append_null();
-                    }
-                });
+        if s.null_count() == s.len() {
+            for _ in 0..s.len() {
+                builder.append_null().unwrap();
+            }
+        } else if s.is_empty() {
+            return;
+        } else {
+            for a in arrays {
+                let values = a.get_values::<T>();
+                // we would like to check if array has no null values.
+                // however at the time of writing there is a bug in append_slice, because it does not update
+                // the null bitmap
+                if s.null_count() == 0 {
+                    builder.append_slice(values).unwrap();
+                } else {
+                    values.iter().enumerate().for_each(|(idx, v)| {
+                        if a.is_valid(idx) {
+                            builder.append_value(*v).unwrap();
+                        } else {
+                            builder.append_null().unwrap();
+                        }
+                    });
+                }
             }
         }
         self.builder.append(true).unwrap();
@@ -548,7 +566,7 @@ pub fn get_list_builder(
 ) -> Box<dyn ListBuilderTrait> {
     macro_rules! get_primitive_builder {
         ($type:ty) => {{
-            let values_builder = PrimitiveArrayBuilder::<$type>::new(value_capacity);
+            let values_builder = PrimitiveBuilder::<$type>::new(value_capacity);
             let builder = ListPrimitiveChunkedBuilder::new(&name, values_builder, list_capacity);
             Box::new(builder)
         }};
@@ -594,7 +612,7 @@ mod test {
 
     #[test]
     fn test_list_builder() {
-        let values_builder = PrimitiveArrayBuilder::<Int32Type>::new(10);
+        let values_builder = PrimitiveBuilder::<Int32Type>::new(10);
         let mut builder = ListPrimitiveChunkedBuilder::new("a", values_builder, 10);
 
         // create a series containing two chunks
