@@ -1,36 +1,106 @@
 use crate::prelude::*;
+use crate::utils::arrow::util::bit_util::unset_bit;
+use crate::utils::CustomIterTools;
 use arrow::array::{
     Array, ArrayData, BooleanArray, LargeStringArray, LargeStringBuilder, PrimitiveArray,
     UInt32Array,
 };
-use arrow::buffer::MutableBuffer;
+use arrow::buffer::{Buffer, MutableBuffer};
+use polars_arrow::buffer::IsValid;
 use std::mem;
 use std::sync::Arc;
 
 /// Take kernel for single chunk without nulls and arrow array as index.
-pub(crate) unsafe fn take_no_null_primitive<T: PolarsNumericType>(
+pub(crate) unsafe fn take_primitive_unchecked<T: PolarsNumericType>(
+    arr: &PrimitiveArray<T>,
+    indices: &UInt32Array,
+) -> Arc<PrimitiveArray<T>> {
+    let array_values = arr.values();
+    let index_values = indices.values();
+    let validity_values = arr
+        .data_ref()
+        .null_buffer()
+        .expect("null buffer should be there");
+    let values_offset = arr.offset();
+    let indices_offset = indices.offset();
+
+    // first take the values, these are always needed
+    let values: AlignedVec<T::Native> = index_values
+        .iter()
+        .map(|idx| *array_values.get_unchecked(*idx as usize))
+        .collect_trusted();
+
+    // the validity buffer we will fill with all valid. And we unset the ones that are null
+    // in later checks
+    // this is in the assumption that most values will be valid.
+    // Maybe we could add another branch based on the null count
+    let num_bytes = indices.len() * std::mem::size_of::<i32>() / 8;
+    let mut validity = MutableBuffer::new(num_bytes).with_bitset(num_bytes, true);
+    let validity_slice = validity.as_slice_mut();
+
+    let arr = if let Some(validity_indices) = indices.data_ref().null_buffer() {
+        index_values.iter().enumerate().for_each(|(i, idx)| {
+            // i is iteration count
+            // idx is the index that we take from the values array.
+            let idx = *idx as usize;
+            if validity_indices.is_null_unchecked(i + indices_offset)
+                || validity_values.is_null_unchecked(idx + values_offset)
+            {
+                unset_bit(validity_slice, i)
+            }
+        });
+        values.into_primitive_array(Some(validity.into()))
+    } else {
+        index_values.iter().enumerate().for_each(|(i, idx)| {
+            let idx = *idx as usize;
+            if validity_values.is_null_unchecked(idx + values_offset) {
+                unset_bit(validity_slice, i)
+            }
+        });
+        values.into_primitive_array(Some(validity.into()))
+    };
+
+    Arc::new(arr)
+}
+
+/// Take kernel for single chunk without nulls and arrow array as index.
+pub(crate) unsafe fn take_no_null_primitive_unchecked<T: PolarsNumericType>(
     arr: &PrimitiveArray<T>,
     indices: &UInt32Array,
 ) -> Arc<PrimitiveArray<T>> {
     assert_eq!(arr.null_count(), 0);
 
-    let data_len = indices.len();
     let array_values = arr.values();
     let index_values = indices.values();
 
-    let mut values = AlignedVec::<T::Native>::with_capacity_aligned(data_len);
     let iter = index_values
         .iter()
         .map(|idx| *array_values.get_unchecked(*idx as usize));
-    values.extend(iter);
 
-    let nulls = indices.data_ref().null_buffer().cloned();
+    // Safety:
+    // indices is trusted length
+    let buffer = Buffer::from_trusted_len_iter(iter);
+    let nulls = indices
+        .data_ref()
+        .null_buffer()
+        .map(|buf| buf.bit_slice(indices.offset(), indices.len()));
 
-    let arr = values.into_primitive_array::<T>(nulls);
-    Arc::new(arr)
+    let data = ArrayData::new(
+        T::DATA_TYPE,
+        indices.len(),
+        nulls.as_ref().map(|_| indices.null_count()),
+        nulls,
+        0,
+        vec![buffer],
+        vec![],
+    );
+    Arc::new(PrimitiveArray::<T>::from(data))
 }
 
 /// Take kernel for single chunk without nulls and an iterator as index.
+///
+/// # Panics
+/// if iterator is not trusted len
 pub(crate) unsafe fn take_no_null_primitive_iter_unchecked<
     T: PolarsNumericType,
     I: IntoIterator<Item = usize>,
@@ -42,13 +112,15 @@ pub(crate) unsafe fn take_no_null_primitive_iter_unchecked<
 
     let array_values = arr.values();
 
-    let av = indices
+    let iter = indices
         .into_iter()
-        .map(|idx| *array_values.get_unchecked(idx))
-        .collect::<AlignedVec<_>>();
+        .map(|idx| *array_values.get_unchecked(idx));
 
-    let arr = av.into_primitive_array::<T>(None);
-    Arc::new(arr)
+    let len = iter.size_hint().0;
+    let buffer = Buffer::from_trusted_len_iter(iter);
+
+    let data = ArrayData::new(T::DATA_TYPE, len, None, None, 0, vec![buffer], vec![]);
+    Arc::new(PrimitiveArray::<T>::from(data))
 }
 
 /// Take kernel for single chunk without nulls and an iterator as index that does bound checks.
@@ -70,6 +142,9 @@ pub(crate) fn take_no_null_primitive_iter<T: PolarsNumericType, I: IntoIterator<
 }
 
 /// Take kernel for a single chunk with null values and an iterator as index.
+///
+/// # Panics
+/// panics if the array does not have nulls
 pub(crate) unsafe fn take_primitive_iter_unchecked<
     T: PolarsNumericType,
     I: IntoIterator<Item = usize>,
@@ -78,9 +153,14 @@ pub(crate) unsafe fn take_primitive_iter_unchecked<
     indices: I,
 ) -> Arc<PrimitiveArray<T>> {
     let array_values = arr.values();
+    let offset = arr.offset();
+    let buf = arr
+        .data_ref()
+        .null_buffer()
+        .expect("null buffer should be there");
 
     let iter = indices.into_iter().map(|idx| {
-        if arr.is_valid(idx) {
+        if buf.is_valid_unchecked(idx + offset) {
             Some(*array_values.get_unchecked(idx))
         } else {
             None
@@ -92,17 +172,26 @@ pub(crate) unsafe fn take_primitive_iter_unchecked<
 }
 
 /// Take kernel for a single chunk with null values and an iterator as index that does bound checks.
+///
+/// # Panics
+/// panics if the array does not have nulls
 pub(crate) fn take_primitive_iter<T: PolarsNumericType, I: IntoIterator<Item = usize>>(
     arr: &PrimitiveArray<T>,
     indices: I,
 ) -> Arc<PrimitiveArray<T>> {
     let array_values = arr.values();
+    let validity = arr
+        .data_ref()
+        .null_bitmap()
+        .as_ref()
+        .expect("bitmap should be set");
+    let offset = arr.offset();
 
     let arr = indices
         .into_iter()
         .map(|idx| {
-            if arr.is_valid(idx) {
-                Some(array_values[idx])
+            if validity.is_set(idx + offset) {
+                array_values.get(idx).copied()
             } else {
                 None
             }
@@ -141,10 +230,15 @@ pub(crate) unsafe fn take_primitive_opt_iter_unchecked<
     indices: I,
 ) -> Arc<PrimitiveArray<T>> {
     let array_values = arr.values();
+    let offset = arr.offset();
+    let buf = arr
+        .data_ref()
+        .null_buffer()
+        .expect("null buffer should be there");
 
     let iter = indices.into_iter().map(|opt_idx| {
         opt_idx.and_then(|idx| {
-            if arr.is_valid(idx) {
+            if buf.is_valid_unchecked(idx + offset) {
                 Some(*array_values.get_unchecked(idx))
             } else {
                 None
@@ -211,15 +305,22 @@ pub(crate) fn take_bool_iter<I: IntoIterator<Item = usize>>(
     arr: &BooleanArray,
     indices: I,
 ) -> Arc<BooleanArray> {
-    let iter = indices.into_iter().map(|idx| {
-        if arr.is_null(idx) {
-            None
-        } else {
-            Some(arr.value(idx))
-        }
-    });
+    if let Some(validity) = arr.data_ref().null_bitmap() {
+        let offset = arr.offset();
+        let iter = indices.into_iter().map(|idx| {
+            if validity.is_set(idx + offset) {
+                Some(arr.value(idx))
+            } else {
+                None
+            }
+        });
 
-    Arc::new(iter.collect())
+        Arc::new(iter.collect())
+    } else {
+        let iter = indices.into_iter().map(|idx| Some(arr.value(idx)));
+
+        Arc::new(iter.collect())
+    }
 }
 
 /// Take kernel for single chunk and an iterator as index.
@@ -227,15 +328,24 @@ pub(crate) unsafe fn take_bool_iter_unchecked<I: IntoIterator<Item = usize>>(
     arr: &BooleanArray,
     indices: I,
 ) -> Arc<BooleanArray> {
-    let iter = indices.into_iter().map(|idx| {
-        if arr.is_null(idx) {
-            None
-        } else {
-            Some(arr.value_unchecked(idx))
-        }
-    });
+    if let Some(buf) = arr.data_ref().null_buffer() {
+        let offset = arr.offset();
+        let iter = indices.into_iter().map(|idx| {
+            if buf.is_valid_unchecked(idx + offset) {
+                Some(arr.value_unchecked(idx))
+            } else {
+                None
+            }
+        });
 
-    Arc::new(iter.collect())
+        Arc::new(iter.collect())
+    } else {
+        let iter = indices
+            .into_iter()
+            .map(|idx| Some(arr.value_unchecked(idx)));
+
+        Arc::new(iter.collect())
+    }
 }
 
 /// Take kernel for single chunk and an iterator as index.
@@ -243,17 +353,27 @@ pub(crate) unsafe fn take_bool_opt_iter_unchecked<I: IntoIterator<Item = Option<
     arr: &BooleanArray,
     indices: I,
 ) -> Arc<BooleanArray> {
-    let iter = indices.into_iter().map(|opt_idx| {
-        opt_idx.and_then(|idx| {
-            if arr.is_null(idx) {
-                None
-            } else {
-                Some(arr.value_unchecked(idx))
-            }
-        })
-    });
+    if let Some(buf) = arr.data_ref().null_buffer() {
+        let offset = arr.offset();
 
-    Arc::new(iter.collect())
+        let iter = indices.into_iter().map(|opt_idx| {
+            opt_idx.and_then(|idx| {
+                if buf.is_valid_unchecked(idx + offset) {
+                    Some(arr.value_unchecked(idx))
+                } else {
+                    None
+                }
+            })
+        });
+
+        Arc::new(iter.collect())
+    } else {
+        let iter = indices
+            .into_iter()
+            .map(|opt_idx| opt_idx.map(|idx| arr.value_unchecked(idx)));
+
+        Arc::new(iter.collect())
+    }
 }
 
 /// Take kernel for single chunk without null values and an iterator as index that may produce None values.
@@ -279,12 +399,21 @@ pub(crate) unsafe fn take_no_null_utf8_iter_unchecked<I: IntoIterator<Item = usi
     Arc::new(iter.collect())
 }
 
+/// # Panics
+///
+/// panics if array has no null data
 pub(crate) unsafe fn take_utf8_iter_unchecked<I: IntoIterator<Item = usize>>(
     arr: &LargeStringArray,
     indices: I,
 ) -> Arc<LargeStringArray> {
+    let offset = arr.offset();
+    let buf = arr
+        .data_ref()
+        .null_buffer()
+        .expect("null buffer should be there");
+
     let iter = indices.into_iter().map(|idx| {
-        if arr.is_null(idx) {
+        if buf.is_null_unchecked(idx + offset) {
             None
         } else {
             Some(arr.value_unchecked(idx))
@@ -305,13 +434,22 @@ pub(crate) unsafe fn take_no_null_utf8_opt_iter_unchecked<I: IntoIterator<Item =
     Arc::new(iter.collect())
 }
 
+/// # Panics
+///
+/// panics if array has no null data
 pub(crate) unsafe fn take_utf8_opt_iter_unchecked<I: IntoIterator<Item = Option<usize>>>(
     arr: &LargeStringArray,
     indices: I,
 ) -> Arc<LargeStringArray> {
+    let offset = arr.offset();
+    let buf = arr
+        .data_ref()
+        .null_buffer()
+        .expect("null buffer should be there");
+
     let iter = indices.into_iter().map(|opt_idx| {
         opt_idx.and_then(|idx| {
-            if arr.is_null(idx) {
+            if buf.is_null_unchecked(idx + offset) {
                 None
             } else {
                 Some(arr.value_unchecked(idx))
@@ -331,15 +469,25 @@ pub(crate) fn take_no_null_utf8_iter<I: IntoIterator<Item = usize>>(
     Arc::new(iter.collect())
 }
 
+/// # Panics
+///
+/// panics if array has no null data
 pub(crate) fn take_utf8_iter<I: IntoIterator<Item = usize>>(
     arr: &LargeStringArray,
     indices: I,
 ) -> Arc<LargeStringArray> {
+    let offset = arr.offset();
+    let validity = arr
+        .data_ref()
+        .null_bitmap()
+        .as_ref()
+        .expect("null buffer should be there");
+
     let iter = indices.into_iter().map(|idx| {
-        if arr.is_null(idx) {
-            None
-        } else {
+        if validity.is_set(idx + offset) {
             Some(arr.value(idx))
+        } else {
+            None
         }
     });
 
@@ -359,7 +507,6 @@ pub(crate) unsafe fn take_utf8(
 
     let mut length_so_far = 0;
     offset_typed[0] = length_so_far;
-    let mut null_count = 0;
 
     let nulls;
 
@@ -398,13 +545,19 @@ pub(crate) unsafe fn take_utf8(
     // THIS BRANCH LEAD TO UB if offset was non zero.
     // also happens with take kernel in arrow
     // offsets in null buffer seem to be the problem.
-    } else if arr.null_count() == 0 && indices.offset() == 0 {
+    } else if arr.null_count() == 0 {
+        let indices_offset = indices.offset();
+        let indices_null_buf = indices
+            .data_ref()
+            .null_buffer()
+            .expect("null buffer should be there");
+
         offset_typed
             .iter_mut()
             .skip(1)
             .enumerate()
             .for_each(|(idx, offset)| {
-                if indices.is_valid(idx) {
+                if indices_null_buf.is_valid_unchecked(idx + indices_offset) {
                     let index = indices.value_unchecked(idx) as usize;
                     let s = arr.value_unchecked(index);
                     length_so_far += s.len() as i64;
@@ -418,15 +571,23 @@ pub(crate) unsafe fn take_utf8(
                 }
                 *offset = length_so_far;
             });
-        nulls = indices.data_ref().null_buffer().cloned();
-        null_count = indices.null_count();
+        nulls = indices
+            .data_ref()
+            .null_buffer()
+            .map(|buf| buf.bit_slice(indices.offset(), indices.len()));
     } else {
         let mut builder = LargeStringBuilder::with_capacity(data_len, length_so_far as usize);
 
+        let arr_offset = arr.offset();
+        let arr_null_buf = arr
+            .data_ref()
+            .null_buffer()
+            .expect("null buffer should be there");
+
         if indices.null_count() == 0 {
             (0..data_len).for_each(|idx| {
                 let index = indices.value_unchecked(idx) as usize;
-                if arr.is_valid(index) {
+                if arr_null_buf.is_valid_unchecked(index + arr_offset) {
                     let s = arr.value_unchecked(index);
                     builder.append_value(s).unwrap();
                 } else {
@@ -434,11 +595,17 @@ pub(crate) unsafe fn take_utf8(
                 }
             });
         } else {
+            let indices_offset = indices.offset();
+            let indices_null_buf = indices
+                .data_ref()
+                .null_buffer()
+                .expect("null buffer should be there");
+
             (0..data_len).for_each(|idx| {
-                if indices.is_valid(idx) {
+                if indices_null_buf.is_valid_unchecked(idx + indices_offset) {
                     let index = indices.value_unchecked(idx) as usize;
 
-                    if arr.is_valid(index) {
+                    if arr_null_buf.is_valid_unchecked(index + arr_offset) {
                         let s = arr.value_unchecked(index);
                         builder.append_value(s).unwrap();
                     } else {
