@@ -7,6 +7,7 @@
 //! are currently supported.
 //!
 use crate::prelude::*;
+use ahash::RandomState;
 pub use arrow::datatypes::DataType as ArrowDataType;
 pub use arrow::datatypes::{
     ArrowNumericType, ArrowPrimitiveType, BooleanType, Date32Type, Date64Type,
@@ -17,11 +18,14 @@ pub use arrow::datatypes::{
     TimestampMicrosecondType, TimestampMillisecondType, TimestampNanosecondType,
     TimestampSecondType, UInt16Type, UInt32Type, UInt64Type, UInt8Type,
 };
+#[cfg(feature = "serde")]
+use serde::{Deserialize, Serialize};
 use std::cmp::Ordering;
 use std::fmt::{Display, Formatter};
 
 pub struct Utf8Type {}
 
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 pub struct ListType {}
 
 pub struct CategoricalType {}
@@ -555,6 +559,11 @@ impl Schema {
         &self.fields
     }
 
+    /// Returns a mutable reference of the vector of `Field` instances
+    pub fn fields_mut(&mut self) -> &mut Vec<Field> {
+        &mut self.fields
+    }
+
     /// Returns an immutable reference of a specific `Field` instance selected using an
     /// offset within the internal `fields` vector
     pub fn field(&self, i: usize) -> Option<&Field> {
@@ -671,3 +680,8 @@ impl From<ArrowSchema> for Schema {
         (&a_schema).into()
     }
 }
+
+#[cfg(feature = "private")]
+pub type PlHashMap<K, V> = hashbrown::HashMap<K, V, RandomState>;
+#[cfg(feature = "private")]
+pub type PlHashSet<V> = hashbrown::HashSet<V, RandomState>;
