@@ -16,7 +16,7 @@ impl DataFrame {
     ///
     /// # Example
     ///
-    /// ```rust
+    /// ```ignore
     ///  use polars_core::prelude::*;
     ///  let s0 = Series::new("a", &[1i64, 2, 3]);
     ///  let s1 = Series::new("b", &[1i64, 1, 1]);
@@ -94,7 +94,10 @@ impl DataFrame {
                 // expand all the other columns based the exploded first column
                 if i == 0 {
                     let row_idx = offsets_to_indexes(offsets, exploded.len());
-                    df = unsafe { df.take_iter_unchecked(row_idx.into_iter()) };
+                    let row_idx = UInt32Chunked::new_from_aligned_vec("", row_idx);
+                    // Safety
+                    // We just created indices that are in bounds.
+                    df = unsafe { df.take_unchecked(&row_idx) };
                 }
                 if exploded.len() == df.height() {
                     df.columns.insert(col_idx, exploded);
@@ -122,7 +125,7 @@ impl DataFrame {
     /// * `id_vars` - String slice that represent the columns to use as id variables.
     /// * `value_vars` - String slice that represent the columns to use as value variables.
     ///
-    /// ```rust
+    /// ```ignore
     ///
     ///  # #[macro_use] extern crate polars_core;
     /// use polars_core::prelude::*;
@@ -207,6 +210,7 @@ mod test {
 
     #[test]
     #[cfg(feature = "dtype-i8")]
+    #[cfg_attr(miri, ignore)]
     fn test_explode() {
         let s0 = Series::new("a", &[1i8, 2, 3]);
         let s1 = Series::new("b", &[1i8, 1, 1]);
@@ -241,6 +245,7 @@ mod test {
     }
 
     #[test]
+    #[cfg_attr(miri, ignore)]
     fn test_melt() {
         let df = df!("A" => &["a", "b", "a"],
          "B" => &[1, 3, 5],
