@@ -414,7 +414,7 @@ impl DataFrame {
 ///
 /// Until described otherwise, the examples in this struct are performed on the following DataFrame:
 ///
-/// ```rust
+/// ```ignore
 /// use polars_core::prelude::*;
 ///
 /// let dates = &[
@@ -1022,7 +1022,7 @@ impl<'df, 'selection_str> GroupBy<'df, 'selection_str> {
 
         macro_rules! finish_agg_opt {
             ($self:ident, $name_fmt:expr, $agg_fn:ident, $agg_col:ident, $cols:ident) => {{
-                let new_name = format![$name_fmt, $agg_col.name()];
+                let new_name = format!($name_fmt, $agg_col.name());
                 let opt_agg = $agg_col.$agg_fn(&$self.groups);
                 if let Some(mut agg) = opt_agg {
                     agg.rename(&new_name);
@@ -1032,7 +1032,7 @@ impl<'df, 'selection_str> GroupBy<'df, 'selection_str> {
         }
         macro_rules! finish_agg {
             ($self:ident, $name_fmt:expr, $agg_fn:ident, $agg_col:ident, $cols:ident) => {{
-                let new_name = format![$name_fmt, $agg_col.name()];
+                let new_name = format!($name_fmt, $agg_col.name());
                 let mut agg = $agg_col.$agg_fn(&$self.groups);
                 agg.rename(&new_name);
                 $cols.push(agg.into_series());
@@ -1057,7 +1057,7 @@ impl<'df, 'selection_str> GroupBy<'df, 'selection_str> {
                         "std" => finish_agg_opt!(self, "{}_std", agg_std, agg_col, cols),
                         "var" => finish_agg_opt!(self, "{}_var", agg_var, agg_col, cols),
                         "count" => {
-                            let new_name = format!["{}_count", agg_col.name()];
+                            let new_name = format!("{}_count", agg_col.name());
                             let mut builder = PrimitiveChunkedBuilder::<UInt32Type>::new(
                                 &new_name,
                                 self.groups.len(),
@@ -1170,20 +1170,20 @@ pub enum GroupByMethod {
 pub fn fmt_groupby_column(name: &str, method: GroupByMethod) -> String {
     use GroupByMethod::*;
     match method {
-        Min => format!["{}_min", name],
-        Max => format!["{}_max", name],
-        Median => format!["{}_median", name],
-        Mean => format!["{}_mean", name],
-        First => format!["{}_first", name],
-        Last => format!["{}_last", name],
-        Sum => format!["{}_sum", name],
+        Min => format!("{}_min", name),
+        Max => format!("{}_max", name),
+        Median => format!("{}_median", name),
+        Mean => format!("{}_mean", name),
+        First => format!("{}_first", name),
+        Last => format!("{}_last", name),
+        Sum => format!("{}_sum", name),
         Groups => "groups".to_string(),
-        NUnique => format!["{}_n_unique", name],
-        Count => format!["{}_count", name],
-        List => format!["{}_agg_list", name],
-        Quantile(quantile) => format!["{}_quantile_{:.2}", name, quantile],
-        Std => format!["{}_agg_std", name],
-        Var => format!["{}_agg_var", name],
+        NUnique => format!("{}_n_unique", name),
+        Count => format!("{}_count", name),
+        List => format!("{}_agg_list", name),
+        Quantile(quantile) => format!("{}_quantile_{:.2}", name, quantile),
+        Std => format!("{}_agg_std", name),
+        Var => format!("{}_agg_var", name),
     }
 }
 
@@ -1198,6 +1198,7 @@ mod test {
 
     #[test]
     #[cfg(feature = "dtype-date32")]
+    #[cfg_attr(miri, ignore)]
     fn test_group_by() {
         let s0 = Date32Chunked::parse_from_str_slice(
             "date",
@@ -1302,6 +1303,7 @@ mod test {
     }
 
     #[test]
+    #[cfg_attr(miri, ignore)]
     fn test_static_groupby_by_12_columns() {
         // Build GroupBy DataFrame.
         let s0 = Series::new("G1", ["A", "A", "B", "B", "C"].as_ref());
@@ -1340,6 +1342,7 @@ mod test {
     }
 
     #[test]
+    #[cfg_attr(miri, ignore)]
     fn test_dynamic_groupby_by_13_columns() {
         // The content for every groupby series.
         let series_content = ["A", "A", "B", "B", "C"];
@@ -1392,6 +1395,7 @@ mod test {
     }
 
     #[test]
+    #[cfg_attr(miri, ignore)]
     fn test_groupby_floats() {
         let df = df! {"flt" => [1., 1., 2., 2., 3.],
                     "val" => [1, 1, 1, 1, 1]
@@ -1406,6 +1410,7 @@ mod test {
     }
 
     #[test]
+    #[cfg_attr(miri, ignore)]
     fn test_groupby_categorical() {
         let mut df = df! {"foo" => ["a", "a", "b", "b", "c"],
                     "ham" => ["a", "a", "b", "b", "c"],
@@ -1431,6 +1436,7 @@ mod test {
     }
 
     #[test]
+    #[cfg_attr(miri, ignore)]
     fn test_groupby_apply() {
         let df = df! {
             "a" => [1, 1, 2, 2, 2],
@@ -1443,6 +1449,7 @@ mod test {
     }
 
     #[test]
+    #[cfg_attr(miri, ignore)]
     fn test_groupby_threaded() {
         for slice in &[
             vec![1, 2, 3, 4, 4, 4, 2, 1],
@@ -1465,6 +1472,7 @@ mod test {
     }
 
     #[test]
+    #[cfg_attr(miri, ignore)]
     fn test_groupby_null_handling() -> Result<()> {
         let df = df!(
             "a" => ["a", "a", "a", "b", "b"],
@@ -1480,6 +1488,7 @@ mod test {
     }
 
     #[test]
+    #[cfg_attr(miri, ignore)]
     fn test_groupby_var() -> Result<()> {
         // check variance and proper coercion to f64
         let df = df![
@@ -1506,6 +1515,7 @@ mod test {
     }
 
     #[test]
+    #[cfg_attr(miri, ignore)]
     fn test_groupby_null_group() -> Result<()> {
         // check if null is own group
         let mut df = df![
