@@ -18,6 +18,10 @@ use crate::utils::{
 };
 
 mod arithmetic;
+#[cfg(feature = "asof_join")]
+pub(crate) mod asof_join;
+#[cfg(feature = "cross_join")]
+pub(crate) mod cross_join;
 pub mod explode;
 pub mod groupby;
 pub mod hash_join;
@@ -25,6 +29,7 @@ pub mod hash_join;
 pub mod row;
 pub mod select;
 mod upstream_traits;
+
 #[cfg(feature = "sort_multiple")]
 use crate::prelude::sort::prepare_argsort;
 use crate::POOL;
@@ -716,11 +721,7 @@ impl DataFrame {
             .par_iter()
             .map(|s| {
                 let mut i = iter.clone();
-                if s.null_count() == 0 {
-                    s.take_iter_unchecked(&mut i)
-                } else {
-                    s.take_iter(&mut i)
-                }
+                s.take_iter_unchecked(&mut i)
             })
             .collect();
         DataFrame::new_no_checks(new_col)
@@ -761,11 +762,7 @@ impl DataFrame {
             .par_iter()
             .map(|s| {
                 let mut i = iter.clone();
-                if s.null_count() == 0 {
-                    s.take_opt_iter_unchecked(&mut i)
-                } else {
-                    s.take_opt_iter(&mut i)
-                }
+                s.take_opt_iter_unchecked(&mut i)
             })
             .collect::<Vec<_>>();
         DataFrame::new_no_checks(new_col)
@@ -809,8 +806,8 @@ impl DataFrame {
             self.columns
                 .par_iter()
                 .map(|s| match s.dtype() {
-                    DataType::Utf8 => s.take_unchecked_threaded(&idx, true).unwrap(),
-                    _ => s.take_unchecked(&idx).unwrap(),
+                    DataType::Utf8 => s.take_unchecked_threaded(idx, true).unwrap(),
+                    _ => s.take_unchecked(idx).unwrap(),
                 })
                 .collect()
         });
@@ -819,7 +816,7 @@ impl DataFrame {
 
     unsafe fn take_unchecked_vectical(&self, indices: &UInt32Chunked) -> Self {
         let n_threads = POOL.current_num_threads();
-        let idxs = split_ca(&indices, n_threads).unwrap();
+        let idxs = split_ca(indices, n_threads).unwrap();
 
         let dfs: Vec<_> = POOL.install(|| {
             idxs.par_iter()
@@ -869,6 +866,8 @@ impl DataFrame {
 
     /// This is the dispatch of Self::sort, and exists to reduce compile bloat by monomorphization.
     fn sort_impl(&self, by_column: Vec<&str>, reverse: Vec<bool>) -> Result<Self> {
+        let first_reverse = reverse[0];
+        let first_by_column = by_column[0];
         let take = match by_column.len() {
             1 => {
                 let s = self.column(by_column[0])?;
@@ -891,7 +890,18 @@ impl DataFrame {
         if std::env::var("POLARS_VERT_PAR").is_ok() {
             return Ok(unsafe { self.take_unchecked_vectical(&take) });
         }
-        Ok(self.take(&take))
+        // Safety:
+        // the created indices are in bounds
+        let mut df = unsafe { self.take_unchecked(&take) };
+        // Mark the first sort column as sorted
+        df.apply(first_by_column, |s| {
+            let mut s = s.clone();
+            let inner = s.get_inner_mut();
+            inner.set_sorted(first_reverse);
+            s
+        })
+        .expect("column is present");
+        Ok(df)
     }
 
     /// Return a sorted clone of this DataFrame.
@@ -1493,7 +1503,7 @@ impl DataFrame {
     ///
     /// # Example
     ///
-    /// ```rust
+    /// ```ignore
     ///
     /// # #[macro_use] extern crate polars_core;
     /// # fn main() {
@@ -1763,6 +1773,7 @@ mod test {
     }
 
     #[test]
+    #[cfg_attr(miri, ignore)]
     fn test_recordbatch_iterator() {
         let df = df!(
             "foo" => &[1, 2, 3, 4, 5]
@@ -1774,6 +1785,7 @@ mod test {
     }
 
     #[test]
+    #[cfg_attr(miri, ignore)]
     fn test_frame_from_recordbatch() {
         let record_batches: Vec<RecordBatch> = create_record_batches();
 
@@ -1786,12 +1798,14 @@ mod test {
     }
 
     #[test]
+    #[cfg_attr(miri, ignore)]
     fn test_select() {
         let df = create_frame();
         assert_eq!(df.column("days").unwrap().eq(1).sum(), Some(1));
     }
 
     #[test]
+    #[cfg_attr(miri, ignore)]
     fn test_filter() {
         let df = create_frame();
         println!("{}", df.column("days").unwrap());
@@ -1800,6 +1814,7 @@ mod test {
     }
 
     #[test]
+    #[cfg_attr(miri, ignore)]
     fn test_filter_broadcast_on_utf8_col() {
         let col_name = "some_col";
         let v = vec!["test".to_string()];
@@ -1814,6 +1829,7 @@ mod test {
     }
 
     #[test]
+    #[cfg_attr(miri, ignore)]
     fn test_filter_broadcast_on_list_col() {
         let s1 = Series::new("", &[true, false, true]);
         let ll: ListChunked = [&s1].iter().copied().collect();
@@ -1826,6 +1842,7 @@ mod test {
     }
 
     #[test]
+    #[cfg_attr(miri, ignore)]
     fn test_sort() {
         let mut df = create_frame();
         df.sort_in_place("temp", false).unwrap();
@@ -1842,6 +1859,7 @@ mod test {
 
     #[test]
     #[cfg(feature = "dtype-u8")]
+    #[cfg_attr(miri, ignore)]
     fn get_dummies() {
         let df = df! {
             "id" => &[1, 2, 3, 1, 2, 3, 1, 1],
@@ -1880,6 +1898,7 @@ mod test {
     }
 
     #[test]
+    #[cfg_attr(miri, ignore)]
     fn drop_duplicates() {
         let df = df! {
             "flt" => [1., 1., 2., 2., 3., 3.],
