@@ -32,7 +32,7 @@ pub(crate) mod private {
     use crate::frame::groupby::pivot::PivotAgg;
     use crate::frame::groupby::GroupTuples;
 
-    use crate::chunked_array::ops::compare_inner::PartialEqInner;
+    use crate::chunked_array::ops::compare_inner::{PartialEqInner, PartialOrdInner};
     use ahash::RandomState;
     use std::borrow::Cow;
 
@@ -49,6 +49,15 @@ pub(crate) mod private {
     }
 
     pub trait PrivateSeries {
+        #[cfg(feature = "asof_join")]
+        fn join_asof(&self, _other: &Series) -> Result<Vec<Option<u32>>> {
+            unimplemented!()
+        }
+
+        fn set_sorted(&mut self, _reverse: bool) {
+            unimplemented!()
+        }
+
         unsafe fn equal_element(
             &self,
             _idx_self: usize,
@@ -61,6 +70,10 @@ pub(crate) mod private {
         fn into_partial_eq_inner<'a>(&'a self) -> Box<dyn PartialEqInner + 'a> {
             unimplemented!()
         }
+        #[allow(clippy::wrong_self_convention)]
+        fn into_partial_ord_inner<'a>(&'a self) -> Box<dyn PartialOrdInner + 'a> {
+            unimplemented!()
+        }
         fn vec_hash(&self, _build_hasher: RandomState) -> AlignedVec<u64> {
             unimplemented!()
         }
@@ -454,7 +467,7 @@ pub trait SeriesTrait:
     ///
     /// # Safety
     ///
-    /// Out of bounds access doesn't Error but will return a Null value
+    /// Out of bounds access doesn't Error but will return a Null value for that element.
     fn take_iter(&self, _iter: &mut dyn Iterator<Item = usize>) -> Series {
         unimplemented!()
     }
@@ -463,7 +476,7 @@ pub trait SeriesTrait:
     ///
     /// # Safety
     ///
-    /// This doesn't check any bounds or null validity.
+    /// This doesn't check any bounds.
     unsafe fn take_iter_unchecked(&self, _iter: &mut dyn Iterator<Item = usize>) -> Series {
         unimplemented!()
     }
@@ -471,7 +484,7 @@ pub trait SeriesTrait:
     /// Take by index if ChunkedArray contains a single chunk.
     ///
     /// # Safety
-    /// This doesn't check any bounds. Null validity is checked.
+    /// This doesn't check any bounds.
     unsafe fn take_unchecked(&self, _idx: &UInt32Chunked) -> Result<Series> {
         unimplemented!()
     }
@@ -480,7 +493,7 @@ pub trait SeriesTrait:
     ///
     /// # Safety
     ///
-    /// This doesn't check any bounds or null validity.
+    /// This doesn't check any bounds.
     unsafe fn take_opt_iter_unchecked(
         &self,
         _iter: &mut dyn Iterator<Item = Option<usize>>,
@@ -492,7 +505,7 @@ pub trait SeriesTrait:
     ///
     /// # Safety
     ///
-    /// Out of bounds access doesn't Error but will return a Null value
+    /// Out of bounds access doesn't Error but will return a Null value for that element
     fn take_opt_iter(&self, _iter: &mut dyn Iterator<Item = Option<usize>>) -> Series {
         unimplemented!()
     }
@@ -501,7 +514,7 @@ pub trait SeriesTrait:
     ///
     /// # Safety
     ///
-    /// Out of bounds access doesn't Error but will return a Null value
+    /// Out of bounds access doesn't Error but will return a Null value for that element.
     fn take(&self, _indices: &UInt32Chunked) -> Series {
         unimplemented!()
     }
@@ -1004,6 +1017,13 @@ pub trait SeriesTrait:
         unimplemented!()
     }
 
+    /// Get a hold to self as `Any` trait reference.
+    /// Only implemented for ObjectType
+    #[cfg(feature = "object")]
+    fn as_any(&self) -> &dyn Any {
+        unimplemented!()
+    }
+
     /// Raise a numeric series to the power of exponent.
     fn pow(&self, _exponent: f64) -> Result<Series> {
         Err(PolarsError::InvalidOperation(
@@ -1037,6 +1057,13 @@ pub trait SeriesTrait:
     fn checked_div(&self, _rhs: &Series) -> Result<Series> {
         unimplemented!()
     }
+
+    #[cfg(feature = "is_first")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "is_first")))]
+    /// Get a mask of the first unique values.
+    fn is_first(&self) -> Result<BooleanChunked> {
+        unimplemented!()
+    }
 }
 
 impl<'a> (dyn SeriesTrait + 'a) {
@@ -1155,7 +1182,7 @@ impl<'a> (dyn SeriesTrait + 'a) {
 pub struct Series(pub Arc<dyn SeriesTrait>);
 
 impl Series {
-    fn get_inner_mut(&mut self) -> &mut dyn SeriesTrait {
+    pub(crate) fn get_inner_mut(&mut self) -> &mut dyn SeriesTrait {
         if Arc::weak_count(&self.0) + Arc::strong_count(&self.0) != 1 {
             self.0 = self.0.clone_inner();
         }
@@ -1469,6 +1496,11 @@ impl Series {
             format!("{:?} is not a floating point datatype", self.dtype()).into(),
         ))
     }
+
+    #[cfg(feature = "dot_product")]
+    pub fn dot(&self, other: &Series) -> Option<f64> {
+        (self * other).sum::<f64>()
+    }
 }
 
 impl Deref for Series {
